#!/usr/bin/php
<?php
// Copyright (C) 2008 - 2015 vortexbox.org
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// Changelog:
//
// Fri Mar 02 2012 <admin@vortexbox.org>
// * Fixed the Blu-Ray code to accept titles with commas in them. 
//		Many thanks to forum user waltonb for this fix!
//
// Sat Feb 25 2012 Ron Olsen <ronolsen@comcast.net>
// * Move putenv calls to start of script so makemkv-con is always invoked as user root.
// * Call getdate() to compute start_time.
//
// Wed Jun 01 2011 Ron Olsen <ronolsen@comcast.net>
// * Call get-cddevice to get the device name instead of hard-coding /dev/sr0.
//
error_reporting(E_ERROR);

// Clean up file names
function fix_file_name($filename)
{	
	$bad_windows_file_characters = array_merge(
			array_map('chr', range(0,31)),
			array('<', '>', ':', '"', '/', "\\", '|', '?', '*', '$'));
	$result = str_replace($bad_windows_file_characters, "", $filename);
	return $result;
}

// Change user and home to root so makemkv-con can get the Blu-ray track info.

putenv("USER=root");
putenv("HOME=/root");

// Cleanup temp files
exec("mkdir -p /storage/tmp/ripper.tmp/");
exec("rm -rf /storage/tmp/ripper.tmp/*");

// Get device name
$cddevice=exec("/opt/vortexbox/get-cddevice");
echo "Using device $cddevice\n";

echo "Analysing Blu-ray track info. This may take a while...\n";

// Time how long it takes to rip movie
$start_time = getdate();
// Extract title and chapter info from the Blu-ray.
exec("/usr/bin/makemkvcon info dev:".$cddevice." -r > /tmp/docd_helper.tmp");
$track_info = file("/tmp/docd_helper.tmp");

// defaults in case we can't parse info data
$movie_title = "Unknown_Title";
$longest_track = 0;
$longest_track_length_seconds = 0;
$length = 0;

for ($i = 0; $i < count($track_info); ++$i) {
  // Find movie title
  $parts = preg_match("/^CINFO:30,0,\"(.*)\"/", $track_info[$i], $t);
  if ($parts != 0) {
    $movie_title = $t[1];
    echo "Disc title: $movie_title\n";
	$movie_title_widnowsfix = fix_file_name($movie_title);
    continue;
  }

  // Find longest track  
  $time_parts = preg_match("/^TINFO:(\d+),9,0,\"(\S+)\"/", $track_info[$i], $t);
  if ($time_parts != 0) {
	preg_match("/(\S+):(\S+):(\S+)/", $t[2], $tparts);
	$length_seconds = ($tparts[1]*3600)+($tparts[2]*60)+$tparts[3];
	echo "Track $t[1] is $length_seconds seconds long.\n";
	if ($length_seconds > $length) {
		$length = $length_seconds;
		$longest_track_length_seconds = $length_seconds;
		$longest_track = $t[1];
	}
  }
}
echo "\nLongest track is track #$longest_track ($longest_track_length_seconds seconds)\n";

// Check if Blu-ray is a TV show archive and enable multi rip
if ($longest_track_length_seconds < 3900) {
	echo "\nNo tracks longer than 60 minutes. Looks like a TV show disk.\n";
	$length = 120;
	$multirip = 1;
}

// Check for user selected multi rip
if (file_exists("/etc/vortexbox/ripdvdlength")) { 
	$length = trim(exec("/bin/cat /etc/vortexbox/ripdvdlength")); 
	$multirip = 1;
}

// Rip the Blu-ray!

// Multi rip
if ($multirip == 1) {
	echo "\nRipping tracks longer than ".$length." seconds.\n\n";
	exec("/usr/bin/makemkvcon mkv --decrypt --progress=-stdout --minlength=".$length." dev:$cddevice all /storage/tmp/ripper.tmp >> /var/log/cdrip.log");

	if (glob("/storage/tmp/ripper.tmp/*.mkv") != false){
		$filecount = count(glob("/storage/tmp/ripper.tmp/*.mkv"));
		echo "Found: $filecount files that were ripped from Blu-ray\n";
	}
	// If disk name already exists create a new directory title
	if (file_exists("/storage/movies/$movie_title_widnowsfix")) {
		$directory_name = "$movie_title_widnowsfix-$start_time[0]";
		echo "Directory already exists, renaming output folder to: $directory_name\n";
	} else {
		$directory_name = $movie_title_widnowsfix;
	}
	exec('mkdir -p "/storage/movies/'.$directory_name.'"');
	exec('for f in /storage/tmp/ripper.tmp/*.mkv; do mv "$f" "/storage/movies/'.$directory_name.'"; done');
	$path = "/storage/movies/".$directory_name;
	$dh = opendir($path);
	$i=1;
	while (($file = readdir($dh)) !== false) {
		if($file != "." && $file != "..") {
		$tracknum = str_pad($i, 2, "0", STR_PAD_LEFT);
		rename($path."//".$file, $path."//".$directory_name."_".$tracknum.".mkv");
		$i++;
		}
	}
	closedir($dh);	
	
// Single rip	
} else {
	echo "\nRipping ".$movie_title." (track ".$longest_track.")\n\n";
	exec("/usr/bin/makemkvcon mkv --decrypt --progress=-stdout --minlength=120 dev:$cddevice ".($longest_track)." /storage/tmp/ripper.tmp >> /var/log/cdrip.log");
	$movie_title_path = "/storage/movies/".$movie_title_widnowsfix.".mkv";
		
	// If movie name already exists create a new directory title
	if (file_exists($movie_title_path)) {
		$movie_title_path = "/storage/movies/".$movie_title_widnowsfix."-".$start_time[0].".mkv";
		echo "Movie already exists, renaming movie to: $movie_title_path\n";
	} 

	exec('mv /storage/tmp/ripper.tmp/*.mkv "'.$movie_title_path.'"');
}

$end_time = getdate();
$difference = $end_time[0] - $start_time[0];
echo "\nRipping took ". intval($difference/60) ." minutes and ". ($difference - (intval($difference/60)*60))." seconds.\n";

if ($fp = fopen("/tmp/docd_helper_dvdname.tmp", 'w')) {
	fwrite($fp, $movie_title);
	fclose($fp); 
} else {
	echo 'Config file unwritable... '; 
}

if (file_exists("/etc/vortexbox/riptomp4")) {
	exec ('mkdir -p "/storage/movies/mp4/'.$directory_name.'"');
	echo "Converting .mkv tracks to .mp4, this can take a while...\n";

	if ($multirip == 1) {	
	  $path = "/storage/movies/".$directory_name;
	  $dh = opendir($path);
	  $i=1;
	  while (($file = readdir($dh)) !== false) {
	   if($file != "." && $file != "..") {
	      echo "Encoding /storage/movies/mp4/".$directory_name."/".substr($file,0,-4).".mp4\n";
		  exec('/usr/bin/ffmpeg -y -i "/storage/movies/'.$directory_name.'/'.$file.'" -loglevel 1 -vcodec mpeg2video -b 5500k -acodec copy -f vob -copyts "/storage/movies/mp4/'.$directory_name.'/'.substr($file,0,-4).'.mp4"');
		  $i++;
	   }
	  }
	  closedir($dh);
	} else {
	 	echo "Encoding /storage/movies/mp4/".$directory_name."/".$directory_name.".mp4\n";
		exec('/usr/bin/ffmpeg -y -i "/storage/movies/'.$directory_name.'.mkv" -loglevel 1 -vcodec mpeg2video -b 5500k -acodec copy -f vob -copyts "/storage/movies/mp4/'.$directory_name.'/'.$movie_title.'.mp4"');
	}
	
	// Create xml for Windows Media Center
	$myFile = "/storage/movies/mp4/".$directory_name."/".$movie_title_widnowsfix.".dvdid.xml";
	$fh = fopen($myFile, 'w') or die("can't open file");
	$stringData = '<?xml version="1.0" encoding="utf-8"?>'."\n";
	fwrite($fh, $stringData);
	$stringData = "<Disc>\n";
	fwrite($fh, $stringData);
	$stringData = "     <Name>".$movie_title."</Name>\n";
	fwrite($fh, $stringData);
	$stringData = "     <ID>".strtoupper($dvdid)."</ID>\n";
	fwrite($fh, $stringData);
	$stringData = "</Disc>\n";
	fwrite($fh, $stringData);
	fclose($fh);

	echo "Finished converting to mp4.\n";
}

if (file_exists("/etc/vortexbox/riptom2ts")) {
	exec ('mkdir -p "/storage/movies/m2ts/'.$directory_name.'"');
	echo "Converting .mkv tracks to .m2ts for PS3, this can take a while...\n";

  	if ($multirip == 1) {	
  	  $path = "/storage/movies/".$directory_name;
 	  $m2path = "/storage/movies/m2ts/".$directory_name;
  	  $dh = opendir($path);
  	  while (($file = readdir($dh)) !== false) {
 	   if(!is_dir($file)) {
 	      echo "Encoding: ".$m2path."/".substr($file,0,-4).".m2ts\n";
		  exec('/usr/bin/ffmpeg -y -i "'.$path.'/'.$file.'" -loglevel 1 -vcodec copy -acodec "'.$m2path.'/'.substr($file,0,-4).'.m2ts"');
  	   }
  	  }
	  closedir($dh);
	} else {
	 	echo "Encoding /storage/movies/m2ts/".$directory_name."/".$directory_name.".m2ts\n";
		exec('/usr/bin/ffmpeg -y -i "/storage/movies/'.$directory_name.'.mkv" -loglevel 1 -vcodec copy -acodec copy "/storage/movies/m2ts/'.$directory_name.'/'.$directory_name.'.m2ts"');
	}
	
	echo "Finished converting to m2ts.\n";
}
?>
