#!/bin/bash
# Copyright (C) 2014 Small Green Computer
#

LIBDIR='/var/lib/squeezeboxserver'

# Were we called correctly?
if [ -z "$1" -o -z "$2" ]; then
  echo "Usage: install-plug.sh <Plug name> <URL>" >&2
  exit 1
fi

PLUGIN_NAME="$1"
PLUGIN_URL="$2"


download_plugin(){
	NAME="$1"
	URL="$2"
	
	TARGETDIR="${LIBDIR}/cache/DownloadedPlugins"
	TARGETFILE="${TARGETDIR}/${NAME}.zip"
	
	TMPFILE="/tmp/plugin_download.tmp"
	
	echo "Downloading ${URL}"
	
	CURL_OPTS=' --connect-timeout 30 --progress-bar --max-time 120'
	
	curl $CURL_OPTS --output "$TMPFILE" "$URL"
	
	# Error downloading?
	if [ $? -gt 0 ]; then
		echo "Error: could not download ${URL}"
		rm "$TMPFILE"
		exit 1
	fi
	
	# Put the file in place..
	mv -f "$TMPFILE" "$TARGETFILE"
	
	if [ ! -f "$TARGETFILE" ]; then
		echo "Error: could not create ${TARGETFILE}."
		rm "$TMPFILE"
		exit 1
	fi
	
	# Fix the permissions
	USER=$(stat -c %U "$TARGETDIR")
	GROUP=$(id -ng $USER)
	chown "${USER}:${GROUP}" "$TARGETFILE"
	
	if [ -f "$TARGETFILE" ]; then
		echo "${TARGETFILE} downloaded successfully!"
	else
		echo "ERROR: ${TARGETFILE} NOT downloaded!"
		exit 1
	fi
	
	return 0
}

fix_prefs(){
	NAME="$1"
	
	#Fixup extensions.prefs
	FIXFILE="${LIBDIR}/prefs/plugin/extensions.prefs"
	
	echo "Fixing up ${FIXFILE}.."

	# make a backup of the prefs file..
	cp -f "$FIXFILE" "${FIXFILE}.bak"

	# Fixup the _ts_plugin: entry with current epoch time
	NEWTIME="$(date --utc '+%s')"
	sed -i -e "s/^_ts_plugin:.*\$/_ts_plugin: ${NEWTIME}/" "$FIXFILE"
	
	# Delete any previous reference to the Plugin..
	sed -i "/ ${NAME}: /d" "$FIXFILE"
	sed -i "/_ts_${NAME}: /d" "$FIXFILE"
	
	# Add the plugin name to the plugin: section..
	sed -i "/^plugin:/a \ \ ${NAME}: 1" "$FIXFILE"
	sed -i -e "s/^plugin:.*\$/plugin:/" "$FIXFILE"
	
	echo "Fixes:"
	cat "$FIXFILE" | egrep '^_ts_plugin: .*$'
	cat "$FIXFILE" | egrep "$NAME"

	# Fixup state.prefs
	FIXFILE="${LIBDIR}/prefs/plugin/state.prefs"

	echo "Fixing up ${FIXFILE}.."

	cp -f "$FIXFILE" "${FIXFILE}.bak"

	# Delete any previous reference to the Plugin..
	sed -i "/^${NAME}:.*\$/d" "$FIXFILE"
	sed -i "/^_ts_${NAME}:.*\$/d" "$FIXFILE"
	
	# Add our 'needs-install' flag
	
sed -i "1,/^_ts_.*\$/ {/^_ts_.*\$/i\
${NAME}: needs-install
}" "$FIXFILE"
	
	
	# Add our _ts_NewPluginName date

sed -i "1,/^_ts_.*\$/ {/^_ts_.*\$/a\
_ts_${NAME}: ${NEWTIME}
}" "$FIXFILE"
	
	echo "Fixes:"
	cat "$FIXFILE" | egrep "$NAME"
}


stop_service(){
	systemctl stop squeezeboxserver.service
}

start_service(){
	systemctl start squeezeboxserver.service
}


stop_service

download_plugin "$PLUGIN_NAME" "$PLUGIN_URL"

fix_prefs "$PLUGIN_NAME"

sleep 2

start_service

sleep 2

CHECK_DIR="${LIBDIR}/cache/InstalledPlugins/Plugins/${PLUGIN_NAME}"

if [ -d "$CHECK_DIR" ]; then
	echo "${PLUGIN_NAME} installed successfully!"
else
	echo "ERROR: ${PLUGIN_NAME} NOT installed!"
fi


